package com.ruoyi.jjh.declaration.regular;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.jjh.declaration.component.TimingRemindService;
import com.ruoyi.jjh.declaration.entity.BmsEnterpriseBasicInfo;
import com.ruoyi.jjh.declaration.entity.BmsTemplateInfo;
import com.ruoyi.jjh.declaration.entity.BmsTemplateRecord;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.declaration.service.IBmsEnterpriseBasicInfoService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateInfoService;
import com.ruoyi.jjh.declaration.service.IBmsTemplateRecordService;
import com.ruoyi.jjh.ent.entity.JContacts;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.entity.JSmartDeclaration;
import com.ruoyi.jjh.ent.entity.JSmartReminders;
import com.ruoyi.jjh.ent.service.JContactsService;
import com.ruoyi.jjh.ent.service.JProjectService;
import com.ruoyi.jjh.ent.service.JSmartDeclarationService;
import com.ruoyi.jjh.ent.service.JSmartRemindersService;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import com.ruoyi.quartz.service.ISysJobService;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 每天固定时间定时任务
 *
 * @author du
 * @since 2024/5/20 10:59
 */
@Configuration
@EnableScheduling
public class TimeChange {

    @Value("${enterpriseUrl}")
    private String enterpriseUrl;

    @Value("${isTiming}")
    private Boolean isTiming;

    @Autowired
    private ISysJobService jobService;
    /**
     * 申报模板
     */
    @Resource
    private IBmsTemplateInfoService bmsTemplateInfoService;

    /**
     * 申报项目记录
     */
    @Resource
    private IBmsTemplateRecordService bmsTemplateRecordService;

    /**
     * 企业联系人
     */
    @Resource
    private JContactsService jContactsService;

    /**
     * 企业信息
     */
    @Autowired
    private IBmsEnterpriseBasicInfoService bmsEnterpriseBasicInfoService;

    /**
     * 智能提醒
     */
    @Resource
    private JSmartRemindersService jSmartRemindersService;

    /**
     * 智能提醒关联表
     */
    @Resource
    private JSmartDeclarationService jSmartDeclarationService;

    /**
     * 企业申报记录
     */
    @Resource
    private IBmsDeclarationRecordsService bmsDeclarationRecordsService;

    /**
     * 短信
     */
    @Resource
    private SmsAlertsService smsAlertsService;

    /**
     * 人员
     */
    @Resource
    private SysUserMapper sysUserMapper;

    /**
     * 申报模板
     */
    @Resource
    private IBmsTemplateInfoService iBmsTemplateInfoService;

    @Resource
    private TimingRemindService timingRemindImpl;
    /**
     * 项目表
     */
    @Resource
    private JProjectService jProjectService;

    /**
     * 根据每天的日期修改模板的状态
     */
    @Scheduled(cron = "1 0 0 * * *")
    private void configureTasks() {
        List<BmsTemplateInfo> list = bmsTemplateInfoService.list();
        for (BmsTemplateInfo items : list) {
            iBmsTemplateInfoService.changeStatus(items);
        }
        bmsTemplateInfoService.updateBatchById(list);
    }

    public void getTime(BmsTemplateRecord items) {
        LocalDate currentDate = LocalDate.now();
        LocalDate startDate = LocalDate.of(items.getStartTime().getYear(), items.getStartTime().getMonth(), items.getStartTime().getDayOfMonth());
        LocalDate endDate = LocalDate.of(items.getEndTime().getYear(), items.getEndTime().getMonth(), items.getEndTime().getDayOfMonth());
        if (currentDate.isBefore(startDate)) {
            items.setStatus(0);
        }
        if (currentDate.isAfter(endDate)) {
            items.setStatus(1);
        }
        if (currentDate.equals(startDate) || currentDate.equals(endDate)) {
            items.setStatus(2);
        }
        if (currentDate.isAfter(startDate) && currentDate.isBefore(endDate)) {
            items.setStatus(2);
        }
    }

    /**
     * 根据每天的日期修改在线模板的状态
     */
    @Scheduled(cron = "1 0 0 * * *")
    private void templateRecordChange() {
        List<BmsTemplateRecord> list = bmsTemplateRecordService.list();
        for (BmsTemplateRecord items : list) {
            getTime(items);
        }
        bmsTemplateRecordService.updateBatchById(list);
    }

    /**
     * 定时更新法人库-法人基本信息（统一社会信用代码）
     */
    @Scheduled(cron = "0 0 2 * * *")
    public void changeEnterpriseInfo() {
        if (isTiming) {
            List<BmsEnterpriseBasicInfo> list = bmsEnterpriseBasicInfoService.list();
            List<BmsEnterpriseBasicInfo> allList = new ArrayList<>();
            for (BmsEnterpriseBasicInfo items : list) {
                String contacts = enterpriseUrl + "/645383/frjbxxtyshxydm";
                HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
                headers.put("Content-Type", "application/json;charset=utf-8");
                //请求体
                JSONObject jsonObject = new JSONObject();//存放参数
                jsonObject.set("tyshxydm", items.getTyshxydm());
                jsonObject.set("pageIndex", 0);
                HttpResponse response = HttpRequest.post(contacts)
                        .addHeaders(headers)
                        .body(jsonObject.toString())
                        .execute();
                if (response.getStatus() == 200) {
                    JSONObject jsonObj = JSONUtil.parseObj(response.body());
                    if ("200".equals(jsonObj.get("code"))) {
                        JSONObject reqData = JSONUtil.parseObj(jsonObj.get("data"));
                        JSONArray arr = reqData.getJSONArray("data"); // 直接获取数组，无需再次解析
                        int total = reqData.getInt("total"); // 只需解析一次total并转换为整数
                        if (total > 0) {
                            // 直接转换第一个元素为BmsEnterpriseBasicInfo对象
                            BmsEnterpriseBasicInfo res = JSONUtil.toBean(arr.getJSONObject(0).toString(), BmsEnterpriseBasicInfo.class);
                            if(items.getType() != 1){
                                res.setQygm("其他");
                            }
                            res.setId(items.getId());
                            res.setMultiple(total);
                            allList.add(res);
                        }
                    }
                }
            }
            bmsEnterpriseBasicInfoService.updateBatchById(allList);
        }
    }

    /**
     * 定时更新法人库-企业联络人员（统一社会信用代码）
     */
    @Scheduled(cron = "0 0 3 * * *")
    public void changeContacts() {
        if (isTiming) {
            List<BmsEnterpriseBasicInfo> list = bmsEnterpriseBasicInfoService.list();
            for (BmsEnterpriseBasicInfo items : list) {
                String contacts = enterpriseUrl + "/645383/qyllrytyshxydm";
                HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
                headers.put("Content-Type", "application/json;charset=utf-8");
                //请求体
                JSONObject jsonObject = new JSONObject();//存放参数
                jsonObject.set("tyshxydm", items.getTyshxydm());
                jsonObject.set("pageIndex", 0);
                HttpResponse response = HttpRequest.post(contacts)
                        .addHeaders(headers)
                        .body(jsonObject.toString())
                        .execute();
                if (response.getStatus() == 200) {
                    JSONObject jsonObj = JSONUtil.parseObj(response.body());
                    if ("200".equals(jsonObj.get("code"))) {
                        JSONObject reqData = JSONUtil.parseObj(jsonObj.get("data"));
                        JSONArray arr = JSONUtil.parseArray(reqData.get("data"));
                        List<JContacts> allList = new ArrayList<>();
                        for (Object o : arr) {
                            JContacts res = JSONUtil.toBean(o.toString(), JContacts.class);
                            allList.add(res);
                        }
                        jContactsService.saveOrUpdateBatch(allList);
                    }
                }
            }
        }
    }


    /**
     * 定时不定期提醒
     */
    @Scheduled(cron = "0 40 1 * * *")
    public void changeNdq() {
        //前端已经去除修改在线模板，所以开始时间和结束时间不会进行更改，未作声明何种状态下删除该条不定期提醒，故不做
        //获取企业的不定时提醒
        JSmartReminders qy = jSmartRemindersService.getById(1);
        //获取政务的不定时提醒
        JSmartReminders zw = jSmartRemindersService.getById(2);
        //查询所有企业申报记录，对应的申报模板不是长期有效并且申报记录未填的，
        List<JSmartDeclaration> bmsDeclarationRecords = bmsDeclarationRecordsService.allNoLimited();
        //如果当前时间在该申报模板的结束时间前自定义的提前天数之前，就去掉
        bmsDeclarationRecords.removeIf(x ->
                !(LocalDate.now().equals(x.getEndTime().minusDays(qy.getDaysAdvance()).toLocalDate()))
        );
        bmsDeclarationRecords.forEach(y -> {
            y.setSmartRemindersId(1L);
            y.setIsRead(1);
            y.setAlertTime(y.getEndTime().minusDays(qy.getDaysAdvance()));
            if (y.getTyshxydm() != null) {
                if (isTiming) {
                    //发送短信
                    timingRemindImpl.getContact(y.getTyshxydm(), "您有1个申报任务即将结束,请前往系统首页查看");
                }
            }
        });

        //可以优化
        List<JProject> jp = jProjectService.lambdaQuery().isNotNull(JProject::getProjectEndTime).list();
        jp.removeIf(x ->
                !(LocalDate.now().equals(x.getProjectEndTime().minusDays(zw.getDaysAdvance()).toLocalDate()))
        );
        jp.forEach(x -> {
            SysUser user = new SysUser();
            user.setUserType("02");
            sysUserMapper.selectUserList(user).forEach(o -> {
                JSmartDeclaration y = new JSmartDeclaration();
                y.setSmartRemindersId(2L);
                y.setIsRead(1);
                y.setAlertTime(x.getProjectEndTime().minusDays(zw.getDaysAdvance()));
                y.setProjectId(x.getId());
                y.setZwId(o.getUserId());
                bmsDeclarationRecords.add(y);
                if (isTiming) {
                    smsAlertsService.enterSms("政府人员-" + o.getNickName(), "您有1个项目即将建设完成,请前往工作台查看。", o.getPhonenumber());
                }
            });
        });
        jSmartDeclarationService.saveBatch(bmsDeclarationRecords);
    }
}
