package com.ruoyi.jjh.ent.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.jjh.ent.entity.response.HonorResponse;
import com.ruoyi.jjh.ent.entity.response.ProjectTrackingResponse;
import com.ruoyi.jjh.ent.service.JDataScreenService;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 数据大屏控制层
 *
 * @author du
 * @since 2024/7/15 14:54
 */
@RestController
@RequestMapping("/jjh/dataScreen")
@Api(tags = "数据大屏")
public class JDataScreenController extends BaseController {

    @Resource
    private JDataScreenService jDataScreenService;

    @Resource
    private SmsAlertsService sms;

    /**
     * 荣誉情况
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "荣誉情况")
    @GetMapping("/honor")
    public AjaxResult getHonor(HonorResponse a) {
        return success(jDataScreenService.getHonor());
    }

    /**
     * 项目追踪情况
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "项目追踪情况")
    @GetMapping("/projectTracking")
    public AjaxResult getProjectTracking(ProjectTrackingResponse a) {
        return success(jDataScreenService.getProjectTracking());
    }

    /**
     * 服务业发展总览
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "服务业发展总览")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "年份1  季度2", required = true),
    })
    @GetMapping("/servicesDevelop")
    public AjaxResult servicesDevelop(Integer type) {
        //数据更新要从新维护SQL
        return success(jDataScreenService.servicesDevelop(type));
    }


    /**
     * 行业纵深情况
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "行业纵深情况")
    @GetMapping("/getIndustryDepth")
    public AjaxResult getIndustryDepth() {
        return success(jDataScreenService.getIndustryDepth());
    }

    /**
     * 所有分行业汇总
     *
     * @return 所有数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov,gov')")
    @ApiOperation(value = "所有分行业汇总")
    @GetMapping("/getAllIndustry")
    public AjaxResult getAllIndustry() {
        return success(jDataScreenService.getAllIndustry());
    }
}
