package com.ruoyi.jjh.ent.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.jjh.ent.entity.JPolicyFile;
import com.ruoyi.jjh.ent.entity.request.JPolicyFileRequest;
import com.ruoyi.jjh.ent.service.JPolicyFileService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.Serializable;

/**
 * 政策文件（j_policy_file）表控制层
 *
 * @author du
 * @since 2024/5/6 11:16
 */
@RestController
@RequestMapping("/jjh/jPolicyFile")
@Api(tags = "政策文件表")
public class JPolicyFileController extends BaseController {

    /**
     * 服务对象
     */
    @Resource
    private JPolicyFileService jPolicyFileService;

    /**
     * 分页查询所有数据
     *
     * @param page               分页对象
     * @param jPolicyFileRequest 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询所有数据", response = JPolicyFile.class)
    @GetMapping
    public AjaxResult selectAll(Page<JPolicyFile> page, JPolicyFileRequest jPolicyFileRequest) {
        return success(jPolicyFileService.page(page, jPolicyFileRequest));
    }


    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据", response = JPolicyFile.class)
    @GetMapping("{id}")
    public AjaxResult selectOne(@PathVariable Serializable id) {
        return success(jPolicyFileService.getById(id));
    }

    /**
     * 新增数据
     *
     * @param jPolicyFile 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    public AjaxResult insert(@RequestBody JPolicyFile jPolicyFile) {
        return success(jPolicyFileService.save(jPolicyFile));
    }

    /**
     * 修改数据
     *
     * @param jPolicyFile 实体对象
     * @return 修改结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "修改数据")
    @PostMapping("/edit")
    public AjaxResult update(@RequestBody JPolicyFile jPolicyFile) {
        return success(jPolicyFileService.updateById(jPolicyFile));
    }

    /**
     * 删除数据
     *
     * @param id 主键结合
     * @return 删除结果
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,other-gov')")
    @ApiOperation(value = "删除数据")
    @PostMapping("/{id}")
    public AjaxResult delete(@PathVariable Long id) {
        return success(jPolicyFileService.removeById(id));
    }
}
